/*
 *
 * by Nakamura Noritsugu
 * from 2000.06.16
 */

#include "ruby.h"
#include <kcc.h>

/* fix me!! */
#define STRING_LEN_TIMES 10

static VALUE mKcc;

static char *code_tbl[] = {"JIS", "JISI", "EUC", "SJIS", "JIS8"};

static char* get_code_name(int code)
{
  switch(code) {
  case JIS:
    return code_tbl[0];
  case ESCI:
    return code_tbl[1];
  case EUC:
    return code_tbl[2];
  case SJIS:
    return code_tbl[3];
  case JIS8:
    return code_tbl[4];
  default:
    rb_raise(rb_eTypeError, "Kcc::JIS|JISI|EUC|SJIS|JIS8 expected");
    break;
  }
}

static VALUE
rb_ext_Kcc_check(int argc, VALUE *argv, VALUE class)
{
  int ret_val;
  VALUE s, extend;

  if (rb_scan_args(argc, argv, "11", &s, &extend) == 1) {
    ret_val = KCC_check(STR2CSTR(s), 0);
  } else {
    if (RTEST(extend)) {
      ret_val = KCC_check(STR2CSTR(s), 1);
    } else {
      ret_val = KCC_check(STR2CSTR(s), 0);
    }
  }

  return INT2FIX(ret_val);
}

static VALUE
rb_ext_Kcc_filter(int argc, VALUE *argv, VALUE class)
{
  int i, ret_val;
  char *strbuf, *outcode_name, *incode_name;
  VALUE sss, outcode, incode, extend, zenkaku, gaiji;
  
  i = rb_scan_args(argc, argv, "24", &sss, &outcode, &incode, &extend, &zenkaku, &gaiji);
  strbuf = ALLOCA_N(char, sizeof(sss) * STRING_LEN_TIMES);
  switch (TYPE(outcode)) {
  case T_STRING:
    outcode_name = STR2CSTR(outcode); break;
  case T_FIXNUM:
    outcode_name = get_code_name(FIX2INT(outcode)); break;
  default:
    rb_raise(rb_eTypeError, "String or Fixnum expected(arg2)");
    break;
  }
  if (i == 2) {
    ret_val = KCC_filter(strbuf, outcode_name, STR2CSTR(sss), "AUTO",
                         0, 0, 0);
  } else {
    switch (TYPE(incode)) {
    case T_STRING:
      incode_name = STR2CSTR(incode); break;
    case T_FIXNUM:
      incode_name = get_code_name(FIX2INT(incode)); break;
    default:
      rb_raise(rb_eTypeError, "String or Fixnum expected(arg3)");
      break;
    }
    if (i == 3){
      ret_val = KCC_filter(strbuf, outcode_name, STR2CSTR(sss), incode_name,
                           0, 0, 0);
    } else if (i == 4) {
      ret_val = KCC_filter(strbuf, outcode_name, STR2CSTR(sss), incode_name,
                           RTEST(extend) ? 1 : 0, 0, 0);
    } else if (i == 5) {
      ret_val = KCC_filter(strbuf, outcode_name, STR2CSTR(sss), incode_name,
                           RTEST(extend) ? 1 : 0, RTEST(zenkaku) ? 1 : 0, 0);
    } else if (i == 6) {
      ret_val = KCC_filter(strbuf, outcode_name, STR2CSTR(sss), incode_name,
                           RTEST(extend) ? 1 : 0, RTEST(zenkaku) ? 1 : 0, RTEST(gaiji) ? 1 : 0);
    }
  }

  if (ret_val == -1) {
    rb_raise(rb_eRuntimeError, "Convert Error!!");
  }
  return rb_str_new2(strbuf);
}

static VALUE
rb_ext_Kcc_tojis(VALUE class, VALUE sss)
{
  int ret_val;
  char *strbuf;

  strbuf = ALLOCA_N(char, sizeof(sss) * STRING_LEN_TIMES);
  ret_val = KCC_filter(strbuf, "JIS", STR2CSTR(sss), "AUTO",
                       0, 0, 0);
  if (ret_val == -1) {
    rb_raise(rb_eRuntimeError, "Convert Error!!");
  }
  return rb_str_new2(strbuf);
}

static VALUE
rb_ext_Kcc_toeuc(VALUE class, VALUE sss)
{
  int ret_val;
  char *strbuf;

  strbuf = ALLOCA_N(char, sizeof(sss) * STRING_LEN_TIMES);
  ret_val = KCC_filter(strbuf, "EUC", STR2CSTR(sss), "AUTO",
                       0, 0, 0);
  if (ret_val == -1) {
    rb_raise(rb_eRuntimeError, "Convert Error!!");
  }
  return rb_str_new2(strbuf);
}

static VALUE
rb_ext_Kcc_tosjis(VALUE class, VALUE sss)
{
  int ret_val;
  char *strbuf;

  strbuf = ALLOCA_N(char, sizeof(sss) * STRING_LEN_TIMES);
  ret_val = KCC_filter(strbuf, "SJIS", STR2CSTR(sss), "AUTO",
                       0, 0, 0);
  if (ret_val == -1) {
    rb_raise(rb_eRuntimeError, "Convert Error!!");
  }
  return rb_str_new2(strbuf);
}


void
Init_kcc(){
  mKcc = rb_define_module("Kcc");

  rb_define_module_function(mKcc, "guess",  rb_ext_Kcc_check, -1);
  rb_define_module_function(mKcc, "filter", rb_ext_Kcc_filter, -1);
  rb_define_module_function(mKcc, "kcc", rb_ext_Kcc_filter, -1);
  rb_define_module_function(mKcc, "tojis",  rb_ext_Kcc_tojis, 1);
  rb_define_module_function(mKcc, "toeuc",  rb_ext_Kcc_toeuc, 1);
  rb_define_module_function(mKcc, "tosjis", rb_ext_Kcc_tosjis, 1);

  rb_define_const(mKcc, "BINARY", INT2FIX(BINARY));
  rb_define_const(mKcc, "ASCII", INT2FIX(ASCII));

  rb_define_const(mKcc, "NONASCII", INT2FIX(NONASCII));
  rb_define_const(mKcc, "JIS", INT2FIX(JIS));
  rb_define_const(mKcc, "ESCI", INT2FIX(ESCI));
  rb_define_const(mKcc, "JISI", INT2FIX(ESCI));
  rb_define_const(mKcc, "ASSUME", INT2FIX(ASSUME));
  rb_define_const(mKcc, "EUC", INT2FIX(EUC));
  rb_define_const(mKcc, "DEC", INT2FIX(DEC));
  rb_define_const(mKcc, "SJIS", INT2FIX(SJIS));
  rb_define_const(mKcc, "JIS8", INT2FIX(JIS8));
  rb_define_const(mKcc, "BIT8", INT2FIX(BIT8));

  rb_define_const(mKcc, "AUTO", rb_str_new2("AUTO"));
}
